/*
* Copyright (c) OpenSim project, http://sim.opensecondlife.org/
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the <organization> nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY <copyright holder> ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
*/

using System;

namespace OpenSim
{
	/// <summary>
	/// Description of PhysicsManager.
	/// </summary>
	public class PhysicsManager
	{
		private SceneGraph _sceneGraph;
		
		public PhysicsManager(SceneGraph sceneGraph)
		{
			_sceneGraph=sceneGraph;
		}
		
		public void UpdatePhysics()
		{
			lock(this._sceneGraph.RootNode)
			{
				for (int i = 0; i < _sceneGraph.RootNode.ChildrenCount; i++)
				{
					if(_sceneGraph.RootNode.GetChild(i).SceneType == 1) //check it is a avatar node
					{
						AvatarData avatar = (AvatarData)_sceneGraph.RootNode.GetChild(i);
						avatar.Position.X += (avatar.Velocity.X * 0.2f);
						avatar.Position.Y += (avatar.Velocity.Y * 0.2f);
						avatar.Position.Z += (avatar.Velocity.Z * 0.2f);
					}
				}
			}
		}
				
	}
}
